#!/usr/bin/env python
# -*- coding: utf-8 -*-
#*****************************************************************************
#    Copyright (C) 2006-2007 Serge Weinstock <serge.weinstock@gmail.com>
#  Distributed under the terms of the BSD License.  The full license is in
#  the file COPYING, distributed as part of this software.
#*****************************************************************************
from qt import * #@UnusedWildImport
from kdecore import * #@UnusedWildImport
from kdeui import * #@UnusedWildImport
from kio import * #@UnusedWildImport
from kfile import * #@UnusedWildImport
from SetFolderIconDlg import SetFolderIconDlg
import os, sys, time, tempfile, kdeutils
from xdg import DesktopEntry

MIN_SIZE = 10
MAX_SIZE = 1000
DEF_SIZE = 128
DELTA_UPDATE = 500
DESK_NAME = ".directory"
THUMB_NAME = ".folder.png"

class SetFolderIconDlgImpl(SetFolderIconDlg):
    """This dialog allows to rename files with the help of regular expressions"""
    def __init__(self, options, parent = None,name = None,modal = 0,fl = 0):
        '''
        Constructor

        @param[in] options the parameters read from the command line arguments
        @param[in] parent the dialog parent
        @param[in] name the dialog name
        @param[in] modal True if the dialog is modal
        @param[in] fl the dialog contruction flags
        '''
        ####################################
        # check the arguments
        ####################################
        if options.count() != 2:
            raise RuntimeError(i18n("Invalid number of arguments"))
        self.original = KURL(options.url(1))
        # if the file is not local. download it
        image_local = QString()
        try:
            if not KIO.NetAccess.download(self.original, image_local, None):
                raise RuntimeError(i18n("Unable to access the file:\n'%1'!\n(%2)").arg(self.original.prettyURL()).arg(KIO.NetAccess.lastErrorString()))
            # let's load the image
            self.image = QImage(image_local)
            if self.image.isNull() or (self.image.width() == 0) or (self.image.height() == 0):
                raise RuntimeError(i18n("Unable to read the file:\n'%1'!").arg(self.original.prettyURL()))
        finally:
            # remove the temp files if they are not local files
            KIO.NetAccess.removeTempFile(image_local)
        ####################################
        # now, we can build the dialog
        ####################################
        cfg = KApplication.kApplication().config()
        cfg.setGroup("SetFolderIcon")
        #
        SetFolderIconDlg.__init__(self,parent,name,modal,fl)
        #
        self.px_scaled.setIconSet(KApplication.kApplication().iconLoader().loadIconSet("viewmag1", KIcon.Small))
        self.px_unscaled.setIconSet(KApplication.kApplication().iconLoader().loadIconSet("view_fit_window", KIcon.Small))
        ssc = cfg.readBoolEntry("showScaled", False)
        self.px_scaled.setOn(ssc)
        self.px_unscaled.setOn(not ssc)
        #
        self.deleteOriginal.setChecked(cfg.readBoolEntry("deleteOriginal", False))
        h = min(self.image.height(), cfg.readUnsignedNumEntry("img_height", DEF_SIZE))
        h = max(MIN_SIZE, min(h, MAX_SIZE))
        w, h = self.__getWidthFromHeight(h)
        self.img_width.setMinValue(MIN_SIZE)
        self.img_width.setMaxValue(MAX_SIZE)
        self.img_width.setValue(w)
        self.img_height.setMinValue(MIN_SIZE)
        self.img_height.setMaxValue(MAX_SIZE)
        self.img_height.setValue(h)
        self.connect(self.img_width, SIGNAL('valueChanged(int)'), self.__iwChanged)
        self.connect(self.img_height, SIGNAL('valueChanged(int)'), self.__ihChanged)
        #
        img = self.image.smoothScale(w, h, QImage.ScaleMin)
        self.pixmap.setImage(QPixmap(img))
        self.pixmap.setStretchMode(ssc)
        self.buttonOriginalSz.setText(i18n("Original size (%1x%1)").arg(self.image.width()).arg(self.image.height()))
        #
        self.folder.setMode(KFile.Directory)
        try:
            self.folder.setKURL(self.original.upURL())
        except AttributeError:
            # some pykde implementations have no mapping for setKURL!
            self.folder.setURL(self.original.upURL().prettyURL())
        #
        self.changing_size = False
        self.last_size_update = time.time()
        #
        self.buttonOk.setIconSet(SmallIconSet("ok"))
        self.buttonCancel.setIconSet(SmallIconSet("cancel"))
        self.buttonHelp.setIconSet(SmallIconSet("help"))
        self.buttonOriginalSz.setIconSet(SmallIconSet("reload"))
        #
        w = cfg.readUnsignedNumEntry("width")
        h = cfg.readUnsignedNumEntry("height")
        if w > 0 and h > 0:
            self.resize(w, h)
    def __getWidthFromHeight(self, h):
        """
        Compute the width and height from a given height
        """
        w = (h * self.image.width())/self.image.height()
        if w > MAX_SIZE:
            w = MAX_SIZE
            h = (w * self.image.height())/self.image.width()
        elif w == 0:
            w = 1
        return w, h
    def __getHeightFromWidth(self, w):
        """
        Compute the width and height from a given height
        """
        h = (w * self.image.height())/self.image.width()
        if h > MAX_SIZE:
            h = MAX_SIZE
            w = (w * self.image.height())/self.image.width()
        elif h == 0:
            h = 1
        return w, h
    def __iwChanged(self, value):
        """
        Callback for image width changes
        """
        if self.changing_size:
            return
        try:
            self.changing_size = True
            QTimer.singleShot(DELTA_UPDATE, self.__onUpdateImg)
            self.last_size_update = time.time()
            w, h = self.__getHeightFromWidth(value)
            self.img_width.setValue(w)
            self.img_height.setValue(h)
        finally:
            self.changing_size = False
    def __ihChanged(self, value):
        """
        Callback for image height changes
        """
        if self.changing_size:
            return
        try:
            self.changing_size = True
            QTimer.singleShot(DELTA_UPDATE, self.__onUpdateImg)
            self.last_size_update = time.time()
            w, h = self.__getWidthFromHeight(value)
            self.img_width.setValue(w)
            self.img_height.setValue(h)
        finally:
            self.changing_size = False
    def __onUpdateImg(self):
        """
        Timer callback for updating the image preview
        """
        # don't update too often
        if time.time() - self.last_size_update < DELTA_UPDATE/1000.:
            return
        w = self.img_width.value()
        h = self.img_height.value()
        img = self.image.smoothScale(w, h, QImage.ScaleMin)
        self.pixmap.setImage(QPixmap(img))
        self.pixmap.setStretchMode(self.pixmap.getStretchMode())
    def onScale(self):
        """
        Callback for the scale button
        """
        self.pixmap.setStretchMode(True)
        self.px_scaled.setOn(True)
        self.px_unscaled.setOn(False)
    def onUnscale(self):
        """
        Callback for the unscale button
        """
        self.pixmap.setStretchMode(False)
        self.px_scaled.setOn(False)
        self.px_unscaled.setOn(True)
    def onOriginalSize(self):
        """
        Callback for the original size button
        """
        self.img_width.setValue(self.image.width())
    def clean(self):
        """
        Removes circular references that can lead to a crash when the dialog is destroyed
        """
        # why ?????
        self.folder.clear()
        self.folder.completionObject().stop()
        self.folder.completionObject().clear()
        self.folder.setURL(QString.null)
    def onOK(self):
        """Close the dialog and apply the user choices"""
        try:
            # save the prefs
            cfg = KApplication.kApplication().config()
            cfg.setGroup("SetFolderIcon")
            cfg.writeEntry("showScaled", self.px_scaled.isOn())
            cfg.writeEntry("deleteOriginal", self.deleteOriginal.isChecked())
            cfg.writeEntry("img_height", self.img_height.value())
            cfg.writeEntry("width", self.width())
            cfg.writeEntry("height", self.height())
            # where do we write stuff?
            dir = KURL(self.folder.url())
            d_img = KURL(dir, THUMB_NAME)
            d_desk = KURL(dir, DESK_NAME)
            # let's first read the desktop file if any
            desktop = DesktopEntry.DesktopEntry()
            desktop_local = QString()
            if KIO.NetAccess.exists(d_desk, True, self):
                if not KIO.NetAccess.download(d_desk, desktop_local, self):
                    raise RuntimeError(i18n("Unable to access the file:\n'%1'!\n(%2)")
                                       .arg(d_desk.prettyURL())
                                       .arg(KIO.NetAccess.lastErrorString()))
                # let's load it
                try:
                    desktop.parse(unicode(desktop_local))
                except DesktopEntry.ParsingError, e:
                    raise RuntimeError(i18n("Unable to parse the desktop file:\n'%1'\n(%2)!")
                                       .arg(d_desk.prettyURL())
                                       .arg(str(e)))
            tmpimg = tmpdesk = ""
            try:
                # let's first write the temporaries files
                tmpfile, tmpimg = tempfile.mkstemp()
                os.close(tmpfile)
                self.pixmap.saveImage(tmpimg, "PNG")
                tmpfile, tmpdesk = tempfile.mkstemp()
                os.close(tmpfile)
                if not desktop.hasGroup(DesktopEntry.DesktopEntry.defaultGroup):
                    desktop.addGroup(DesktopEntry.DesktopEntry.defaultGroup)
                desktop.set("Icon", "./"+THUMB_NAME) # why does KDE need the extra "./" ?
                desktop.write(tmpdesk)
                # let's move them to their final destination
                if not KIO.NetAccess.file_move(KURL(tmpimg), d_img, -1, True, False, self):
                    raise RuntimeError(i18n("Unable to write the file:\n'%1'!\n(%2)")
                                       .arg(d_img.prettyURL())
                                       .arg(KIO.NetAccess.lastErrorString()))
                if not KIO.NetAccess.file_move(KURL(tmpdesk), d_desk, -1, True, False, self):
                    raise RuntimeError(i18n("Unable to write the file:\n'%1'!\n(%2)")
                                       .arg(d_img.prettyURL())
                                       .arg(KIO.NetAccess.lastErrorString()))
            finally:
                try: os.unlink(tmpimg)
                except: pass
                try: os.unlink(tmpdesk)
                except: pass
            if self.deleteOriginal.isChecked():
                if not KIO.NetAccess.del_(self.original):
                    raise RuntimeError(i18n("Unable to delete the file:\n'%1'!\n(%2)")
                                       .arg(self.original.prettyURL())
                                       .arg(KIO.NetAccess.lastErrorString()))
            # banzai!
            self.accept()
        except:
            error = sys.exc_info()
            kdeutils.detailedError(None, None, error)

def run(options):
    """
    Create and launch the dialog and do the associated actions.
    param[in] options the program options
    """
    dialog = None
    kapp = None
    res = 1
    try:
        kapp = KApplication.kApplication()
        dialog = SetFolderIconDlgImpl(options)
        kapp.setMainWidget(dialog)
        dialog.show()
        res = kapp.exec_loop()
    except:
        error = sys.exc_info()
        kdeutils.detailedError(None, None, error)
    if dialog:
        dialog.clean()
        del dialog
    return res
